@@ -13,11 +13,15 @@
 -- You should have received a copy of the GNU General Public License    
 -- along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-blueprint.steeringsensitivity = {0.0005, 0.004, 1.35}
+blueprint.anchor = {0.5, 0.75}
+
+blueprint.steeringsensitivity = {0.1, 0.08, 2.2}
 blueprint.steeringlag = 0.05
 
-blueprint.throttlesensitivity = 0.0013
+blueprint.throttlesensitivity = 0.0017
 blueprint.throttlelag = 0.1
+blueprint.limiterthreshold = 15000
+blueprint.limiterhysteresis = 300
 
 blueprint.rearbrakesensitivity = 1.2
 blueprint.rearbrakelag = 0.05
@@ -25,6 +29,9 @@ blueprint.rearbrakelag = 0.05
 blueprint.frontbrakelag = 0.05
 blueprint.frontbrakesensitivity = 3.5
 
-blueprint.riderroll = {760, 34, 0.55}
-blueprint.rideryaw = {120, 13.5, 0}
-blueprint.ridertranslate = {100000, 100, 0.4}
+blueprint.riderroll = {380, 34}
+blueprint.rideryaw = {820, 90}
+blueprint.ridercoupling = {130, 5}
+
+blueprint.loadregion = {100, 0}
+blueprint.sideslipregion = {0.03, 0.1}
\ No newline at end of file
