@@ -1,4 +1,4 @@
--- Copyright (C) 2010-2011 Papavasileiou Dimitris                             
+-- Copyright (C) 2010-2011 Papavasileiou Dimitris                           
 --                                                                      
 -- This program is free software: you can redistribute it and/or modify 
 -- it under the terms of the GNU General Public License as published by 
@@ -21,32 +21,31 @@ require "moremath"
 require "automotive"
 require "morewidgets"
 
-local p_0 = {-1.41 / 2, 0, 0}
+local M = blueprint.masses
+local P = blueprint.points
+local L = blueprint.locks
+local theta = blueprint.headangle
+
+-- Position the vehicle so that the rear chassis
+-- center of mass is just behind the start line.
+-- This will start timing the lap as soon as the
+-- vehicle launches.
+
+local p_0 = {-P[7][1] - 0.01, 0, 0}
 local R_0 = transforms.euler (0, 0, options.reverse and 180 or 0)
 local V_0 = 0
 
 local twistaxis = {math.cos(blueprint.headangle), 0, math.sin(blueprint.headangle)}
 local steeraxis = {-math.sin(blueprint.headangle), 0, math.cos(blueprint.headangle)}
 
-local M = blueprint.masses
-local P = blueprint.points
-local L = blueprint.locks
-local theta = blueprint.headangle
-
-if options.replay then
-   blackboxes = assert(loadfile(options.replay))()
-   print (string.format ("  %s (%d channels)",
-			 options.replay, #blackboxes))
-else
-   blackboxes = {{}, {}, {}, {}, {}}
-end
-
 -- Create the machine.
 
 local root = bodies.system {
-   -- The rear frame.
+   -- The swingarm.
+
+   swingarm = bodies.point {
+      name = "Swingarm",
 
-   rear = bodies.point {
       position = math.add (p_0, math.transform (R_0, P[12])),
       orientation = R_0,
       velocity = math.transform (R_0, {V_0, 0, 0}),
@@ -60,10 +59,14 @@ local root = bodies.system {
       },
 
       spindle = joints.hinge {
+	 name = "Rear wheel spindle",
+
 	 axis = math.transform (R_0, {0, 1, 0}),
 	 anchor = math.add (p_0, math.transform (R_0, P[6])),
 	 
 	 wheel = automotive.wheel {
+	    name = "Rear wheel",
+
 	    position = math.add (p_0, math.transform (R_0, P[6])),
 	    orientation = R_0,
 	    velocity = math.transform (R_0, {V_0, 0, 0}),
@@ -88,19 +91,27 @@ local root = bodies.system {
 	    longitudinal = blueprint.rearlongitudinal,  	    
 	    lateral = blueprint.rearlateral,
 	    moment = blueprint.rearmoment,
+
+	    scaling = blueprint.rearscaling,
 	 },
       },
 
       upperpivot = joints.hinge {
+	 name = "Swing-arm pivot",
+
 	 axis = math.transform (R_0, {0, 1, 0}),
 	 anchor = math.add (p_0, math.transform (R_0, P[10])),
       },      
 
       lowerpivot = joints.hinge {
+	 name = "Suspension linkage pivot",
+
 	 axis = math.transform (R_0, {0, 1, 0}),
 	 anchor = math.add (p_0, math.transform (R_0, P[16])),
 	 
 	 arm = bodies.point {
+	    name = "Suspension linkage arm",
+
 	    position = math.add (p_0, math.transform (R_0, P[16])),
 	    velocity = math.transform (R_0, {V_0, 0, 0}),
 
@@ -113,10 +124,14 @@ local root = bodies.system {
 	    },
 
 	    pivot = joints.hinge {
+	       name = "Suspension linkage pivot",
+
 	       axis = math.transform (R_0, {0, 1, 0}),
 	       anchor = math.add (p_0, math.transform (R_0, P[14])),
 	       
 	       arm = bodies.point {
+		  name = "Suspension linkage arm",
+
 		  position = math.add (p_0, math.transform (R_0, P[14])),
 		  velocity = math.transform (R_0, {V_0, 0, 0}),
 
@@ -129,15 +144,20 @@ local root = bodies.system {
 		  },
 
 		  rightpivot = joints.hinge {
+		     name = "Suspension linkage pivot",
+
 		     axis = math.transform (R_0, {0, 1, 0}),
 		     anchor = math.add (p_0, math.transform (R_0, P[13])),
 		  },
 
 		  leftpivot = joints.hinge {
+		     name = "Rear damper pivot",
+
 		     axis = math.transform (R_0, {0, 1, 0}),
 		     anchor = math.add (p_0, math.transform (R_0, P[15])),
 		     
 		     lowershock = bodies.point {
+			name = "Rear damper rod",
 			position = math.add (p_0, math.transform (R_0, P[15])),
 			velocity = math.transform (R_0, {V_0, 0, 0}),
 
@@ -150,6 +170,8 @@ local root = bodies.system {
 			},
 
 			slider = joints.slider {
+			   name = "Rear damper piston",
+
 			   axis = math.transform (R_0, {-0.13619, 0, -0.99068}),
 
 			   stops = {blueprint.rearlocks, {0, 0.2}, 0.3},
@@ -182,6 +204,7 @@ local root = bodies.system {
 			   	  end,
 
 			   uppershock = bodies.point {
+			      name = "Rear damper cylinder",
 			      position = math.add (p_0, math.transform (R_0, P[11])),
 			      velocity = math.transform (R_0, {V_0, 0, 0}),
 			
@@ -194,6 +217,8 @@ local root = bodies.system {
 			      },
 
 			      pivot = joints.hinge {
+				 name = "Rear damper pivot",
+
 				 axis = math.transform (R_0, {0, 1, 0}),
 				 anchor = math.add (p_0, math.transform (R_0, P[11])),
 			      },
@@ -207,15 +232,17 @@ local root = bodies.system {
       },
 
       link = function (self)
-		self.upperpivot.reference = self.parent.main
-		self.lowerpivot.arm.pivot.arm.rightpivot.reference = self.parent.main
-		self.lowerpivot.arm.pivot.arm.leftpivot.lowershock.slider.uppershock.pivot.reference = self.parent.main
+		self.upperpivot.reference = self.parent.rear
+		self.lowerpivot.arm.pivot.arm.rightpivot.reference = self.parent.rear
+		self.lowerpivot.arm.pivot.arm.leftpivot.lowershock.slider.uppershock.pivot.reference = self.parent.rear
 	     end,
    },
 
    -- The front frame.
 
    front = bodies.point {
+      name = "Front frame",
+
       position = math.add (p_0, math.transform (R_0, P[3])),
       orientation = math.concatenate(R_0, transforms.euler (0, -math.deg(theta), 0)),
       velocity = math.transform (R_0, {V_0, 0, 0}),
@@ -228,11 +255,21 @@ local root = bodies.system {
 	  M[1][6], M[1][7], M[1][4]}
       },
 
-      servo = motors.angular {
-      	 axes = {math.transform(R_0, steeraxis)},
+      servo = motors.euler {
+	 name = "Steering servo",
+
+      	 axes = {math.transform(R_0, {0, 0, 1}),
+		 nil,
+		 math.transform(R_0, {1, 0, 0})},
+
+	 stops = {
+	    {{0, 0}, physics.spring (unpack(blueprint.ridercoupling)), 0},
+	 },
       },
 
       stem = joints.universal {
+	 name = "Steering stem",
+
 	 anchor = math.add (p_0, math.transform (R_0, P[2])),
 	 axes = {math.transform (R_0, steeraxis),
 		 math.transform (R_0, twistaxis)},
@@ -257,8 +294,9 @@ local root = bodies.system {
       },
 
       slider = joints.slider {
-	 axis = math.transform (R_0, steeraxis),
+	 name = "Front damper piston",
 
+	 axis = math.transform (R_0, steeraxis),
 	 stops = {blueprint.frontlocks, {0, 0.2}, 0.3},
 	 
 	 stiffness = blueprint.frontstiffness,
@@ -286,6 +324,8 @@ local root = bodies.system {
 	 	end,
 
 	 legs = bodies.point {
+	    name = "Fork tubes",
+
 	    position = math.add (p_0, math.transform (R_0, P[4])),
 	    orientation = math.concatenate(R_0, transforms.euler (0, -math.deg(theta), 0)),
 	    velocity = math.transform (R_0, {V_0, 0, 0}),
@@ -303,10 +343,14 @@ local root = bodies.system {
 	    },
 
 	    spindle = joints.hinge {
+	       name = "Front wheel spindle",
+
 	       axis = math.transform (R_0, {0, 1, 0}),
 	       anchor = math.add (p_0, math.transform (R_0, P[5])),
 
 	       wheel = automotive.wheel {
+		  name = "Front wheel",
+
 	       	  position = math.add (p_0, math.transform (R_0, P[5])),
 	       	  orientation = R_0,
 		  velocity = math.transform (R_0, {V_0, 0, 0}),  
@@ -331,20 +375,24 @@ local root = bodies.system {
 		  longitudinal = blueprint.frontlongitudinal,  	    
 		  lateral = blueprint.frontlateral,
 		  moment = blueprint.frontmoment,
+
+		  scaling = blueprint.frontscaling,
 	       },
 	    },
 	 },
       },
       
       link = function (self)
-		self.stem.reference = self.parent.main
-		self.servo.reference = self.parent.main.pelvis.butt.waist.torso
+		self.stem.reference = self.parent.rear
+		self.servo.reference = self.parent.rear.waist.torso
 	     end,
    },
 
-   -- The main frame including rider.
+   -- The rear frame including rider.
+
+   rear = bodies.box {
+      name = "Rear frame",
 
-   main = bodies.box {
       position = math.add (p_0, math.transform (R_0, P[7])),
       orientation = R_0,
       velocity = math.transform (R_0, {V_0, 0, 0}),
@@ -374,6 +422,7 @@ local root = bodies.system {
       	       -- print ("Sidestand on.")
 
       	       self.ancestry[2].standmotor = motors.euler {
+		  name = "Side-stand",
       		  axes = {transforms.fromnode (self.ancestry[2], {1, 0, 0}),
       			  transforms.fromnode (self.ancestry[2], {0, 1, 0}),
       			  transforms.fromnode (self.ancestry[2], {0, 0, 1})},
@@ -392,77 +441,36 @@ local root = bodies.system {
       		     end,
       	 }, 
       },
-      
-      pelvis = joints.slider {
-	 axis = math.transform (R_0, {0, 1, 0}),
+
+      waist = joints.universal {
+	 name = "Rider waist",
+
 	 anchor = math.add (p_0, math.transform (R_0, P[8])),
+	 axes = {math.transform (R_0, {1, 0, 0}),
+		 math.transform (R_0, {0, 0, 1})},
 
-	 translate = blueprint.ridertranslate,
+	 stops = {
+	    {{0, 0}, physics.spring (unpack(blueprint.riderroll)), 0},
+	    {{0, 0}, physics.spring (unpack(blueprint.rideryaw)), 0},
+	 },
 
-	 step = function (self)
-		   local gamma
+	 torso = bodies.box {
+	    name = "Rider upper body",
 
-		   gamma = parts.rearwheel.state[1]
-			 
-		   self.stops = {{self.translate[3] * gamma,
-				  self.translate[3] * gamma},
-				 physics.spring (self.translate[1], 
-						 self.translate[2]), 0}
-		end,
-
-	 butt = bodies.box {
-      	    position = math.add (p_0, math.transform (R_0, P[8])),
-      	    orientation = R_0,
+	    position = math.add (p_0, math.transform (R_0, P[9])),
+	    orientation = R_0,
+	    velocity = math.transform (R_0, {V_0, 0, 0}),
 
 	    isvehicle = true,
 
-	    size = math.scale({0.2, 0.24, 0.1}, 1),
-
-      	    mass = {
-      	       1e-3,
-      	       {0, 0, 0},
-      	       {1e-6, 0, 0,
-      		0, 1e-6, 0,
-      		0, 0, 1e-6}
-      	    },
-
-	    waist = joints.universal {
-	       anchor = math.add (p_0, math.transform (R_0, P[8])),
-	       axes = {math.transform (R_0, {1, 0, 0}),
-		       math.transform (R_0, {0, 0, 1})},
-
-	       roll = blueprint.riderroll,
-	       yaw = blueprint.rideryaw,
-
-	       step = function (self)
-			 local gamma
-
-			 gamma = parts.rearwheel.state[1]
-			 
-			 self.stops = {
-			    {{self.roll[3] * gamma, self.roll[3] * gamma},
-			     physics.spring (self.roll[1], self.roll[2]), 0},
-			    {{self.yaw[3] * gamma, self.yaw[3] * gamma},
-			     physics.spring (self.yaw[1], self.yaw[2]), 0}}
-		      end,
-
-	       torso = bodies.box {
-		  position = math.add (p_0, math.transform (R_0, P[9])),
-		  orientation = R_0,
-		  velocity = math.transform (R_0, {V_0, 0, 0}),
-
-		  isvehicle = true,
-
-		  size = math.scale({0.6, 0.24, 0.15}, 1),
-		  
-		  mass = {
-		     M[4][1],
-		     {0, 0, 0},
-		     {M[4][2], M[4][5], M[4][6], 
-		      M[4][5], M[4][3], M[4][7], 
-		      M[4][6], M[4][7], M[4][4]}
-		  },
-	       },
+	    size = math.scale({0.6, 0.24, 0.15}, 1),
+	    
+	    mass = {
+	       M[4][1],
+	       {0, 0, 0},
+	       {M[4][2], M[4][5], M[4][6], 
+		M[4][5], M[4][3], M[4][7], 
+		M[4][6], M[4][7], M[4][4]}
 	    },
 	 },
       },
@@ -476,12 +484,101 @@ local root = bodies.system {
 	    orientation = transforms.relue (90, 90, -90),
 	 },
       } or frames.observer {
-	 position = options.sideview and {0, -3, 0} or {0.4, 0, 1},
-	 orientation = options.sideview and transforms.euler (-90, 0, 90) or
-	                                    transforms.relue (90, 90, -90),
+	 position = {0.4, 0, 1},
+	 orientation = transforms.relue (90, 90, -90),
+      },
+
+      engine = automotive.fourstroke {
+	 name = "Engine",
+
+      	 axis = math.transform (R_0, {0, 1, 0}),
+      	 anchor = math.add (p_0, math.transform (R_0, P[7])),
+
+	 displacement = blueprint.displacement,
+	 cylinders = blueprint.cylinders,
+	 intake = blueprint.intake,
+	 volumetric = blueprint.volumetric,
+	 thermal = blueprint.thermal,
+	 exchange = blueprint.exchange,
+	 friction = blueprint.friction,
+
+	 peak = blueprint.peak,
+
+	 flywheel = bodies.point {
+	    name = "Flywheel",
+
+	    position = math.add (p_0, math.transform (R_0, P[7])),
+	    orientation = R_0,
+	    spin = math.transform (R_0,
+				   {0, units.rotationsperminute(1300), 0}),
+
+      	    mass = {
+      	       1e-3,
+      	       {0, 0, 0},
+      	       {1e-6, 0, 0,
+      		0, blueprint.inertia, 0,
+      		0, 0, 1e-6}
+      	    },
+
+	    clutch = joints.hinge {
+	       name = "Clutch",
+
+	       axis = math.transform (R_0, {0, 1, 0}),
+	       anchor = math.add (p_0, math.transform (R_0, P[7])),
+	       motor = {0, 1 / 0},
+ 
+	       mainshaft = bodies.point {
+		  name = "Mainshaft",
+
+		  position = math.add (p_0, math.transform (R_0, P[7])),
+		  orientation = R_0,
+		  spin = math.transform (R_0,
+					 {0, units.rotationsperminute(1300), 0}),
+
+		  mass = {
+		     1e-3,
+		     {0, 0, 0},
+		     {1e-6, 0, 0,
+		      0, 1e-6, 0,
+		      0, 0, 1e-6}
+		  },
+
+		  gearbox = joints.gearing {
+		     name = "Gearbox",
+
+		     ratio = nil,
+		     
+		     link = function (self)
+			       self.reference = self.ancestry[5].countershaft.sprocket
+			    end,
+		  }
+	       }, 
+	    },
+	 },
+
+	 exhaust = resources.loop "slipstream/waves/engine.lc" {
+	    reference = 1 / 0,
+
+	    gain = 0,
+
+	    prepare = function (self)
+	 		 local m
+	 
+	 		 m = (math.clamp((1256.6 - self.parent.state[2]) / 418.88,
+	 				 0, 1) *
+	 		      math.clamp((self.parent.state[2] - 418.88) / 418.88,
+	 				 0, 1))
+
+	 		 self.gain = (0.4 + 0.6e-5 * self.parent.state[3])--*m
+	 
+	 		 self.pitch = self.parent.state[2] / 837.76
+	 	      end
+	 },
       },
 
       countershaft = joints.hinge {
+	 name = "Countershaft",
+
       	 -- The countershaft axis location is calculated as an offset
       	 -- from the swingarm pivot location.
 
@@ -505,53 +602,12 @@ local root = bodies.system {
 	       radii = blueprint.sprockets,
 	 
       	       link = function (self)
-      	    		 self.reference = self.ancestry[4].rear.spindle.wheel--.damper.sprocket
+      	    		 self.reference = self.ancestry[4].swingarm.spindle.wheel--.damper.sprocket
       	    	      end,
       	    }
       	 },
       },
 
-      engine = automotive.fourstroke {
-      	 axis = math.transform (R_0, {0, 1, 0}),
-      	 anchor = math.add (p_0, math.transform (R_0, P[7])),
-
-	 displacement = blueprint.displacement,
-	 cylinders = blueprint.cylinders,
-	 intake = blueprint.intake,
-	 volumetric = blueprint.volumetric,
-	 thermal = blueprint.thermal,
-	 exchange = blueprint.exchange,
-	 friction = blueprint.friction,
-	 inertia = blueprint.inertia,
-	 primary = blueprint.primary,
-
-	 gearbox = blueprint.gearbox,
-
-	 exhaust = resources.loop "slipstream/waves/engine.lc" {
-	    reference = 1 / 0,
-
-	    gain = 0,
-
-	    prepare = function (self)
-			 local m
-			 
-			 m = (math.clamp((1256.6 - self.parent.state[2]) / 418.88,
-					 0, 1) *
-			      math.clamp((self.parent.state[2] - 418.88) / 418.88,
-					 0, 1))
-
-			 self.gain = (0.4 + 0.6e-5 * self.parent.state[3])--*m
-			 
-			 self.pitch = self.parent.state[2] / 837.76
-		      end
-	 },
-	     
-	 link = function (self)
---      	   self.reference = self.ancestry[2].rear.spindle.wheel
-		   self.reference = self.parent.countershaft.sprocket
-		end,
-      },
-
       aerodynamics = frames.node {
 	 drag = blueprint.drag,
 	 lift = blueprint.lift,
@@ -582,31 +638,19 @@ local root = bodies.system {
    },
 
    control = slipstream.control {
-      unlink = function (self)
-		  if options.record then
-		     file = io.open (options.record, "w")
-		     file:write ("return \n" .. string.serialize(blackboxes))
-		     file:close ()
-
-		     print(string.format ("Dumped %d black box channels to %s.",
-					  #blackboxes, options.record))
-		  end
-	       end,
-
       rubberband = slipstream.rubberband {
+	 anchor = blueprint.anchor or {0.5, 0.75},
+
 	 -- Steering control.
 
 	 steering = slipstream.cascade {
 	    axis = 1,
 
-	    frames.node {
-	       input = 0,
-	       output = 0,
-
+	    slipstream.module {
 	       update = function (self)
 			   local V, gamma
 			   
-			   V = math.length(parts.chassis.velocity)
+			   V = math.length(parts.rearframe.velocity)
 			   gamma = math.abs(parts.rearwheel.state[1])
 			   
 			   self.output = (blueprint.steeringsensitivity[1] +
@@ -618,19 +662,30 @@ local root = bodies.system {
 	    },
 
 	    slipstream.lag {lag = blueprint.steeringlag},
-	    slipstream.blackbox {entries = blackboxes[1]},
-
-	    frames.node {
-	       input = 0,
-	       output = 0,
+	    slipstream.blackbox {channel = 1},
 
+	    slipstream.module {
 	       update = function (self)
-			   if math.length (parts.chassis.velocity) <
+			   local beta = math.abs(parts.frontwheel.state[3])
+			   local N = parts.frontwheel.state[4]
+			   local a = blueprint.loadregion or {100, 0}
+			   local b = blueprint.sideslipregion or {0.25, 0.3}
+
+			   if N < a[1] then
+			      self.input = self.input * math.clamp((a[2] - N) / (a[2] - a[1]), 0, 1)
+			   end
+
+			   if beta > b[1] then
+			      self.input = self.input * math.clamp((b[2] - beta) / (b[2] - b[1]), 0, 1)
+			   end
+			   
+			   if math.length (parts.rearframe.velocity) >
 			      units.kilometersperhour(5) then
-			      parts.servo.motor = {{0, 1}}
-			   else
-			      parts.servo.motor = {{self.input / 0,
-			      			    math.abs(self.input)}}
+			      physics.addtorque (parts.servo.parent,
+						 math.scale(parts.servo.axes[1], self.input))
+
+			      physics.addtorque (parts.servo.reference,
+			      			 math.scale(parts.servo.axes[1], -self.input))
 			   end
 		        end,
 	    },
@@ -641,10 +696,7 @@ local root = bodies.system {
 	 throttle = slipstream.cascade {
 	    axis = 2,
 
-	    frames.node {
-	       input = 0,
-	       output = 0,
-
+	    slipstream.module {
 	       update = function (self)
 			   self.output = tonumber(options.throttle) or
 			   math.clamp(self.input *
@@ -654,26 +706,49 @@ local root = bodies.system {
 
 	    slipstream.lag {lag = blueprint.throttlelag},
 
-	    slipstream.blackbox {entries = blackboxes[2]},
-	    frames.node {
-	       input = 0,
-	       output = 0,
-
+	    slipstream.blackbox {channel = 2},
+	    slipstream.module {
 	       update = function (self)
 			   parts.engine.throttle = self.input
 		        end,
 	    },
 	 },
 
+	 limiter = slipstream.cascade {
+	    slipstream.module {
+	       engaged = false,
+
+	       update = function (self)
+			   if blueprint.limiterthreshold and
+			      blueprint.limiterhysteresis then
+			      limit = units.rotationsperminute(blueprint.limiterthreshold + (self.engaged and -1 or 1) * blueprint.limiterhysteresis)
+			   else
+			      limit = 1 / 0
+			   end
+			   
+			   if parts.engine.state[2] > limit then
+			      self.output = 0
+			      self.engaged = true
+			   else
+			      self.output = 1
+			      self.engaged = false
+			   end
+			end
+	    },
+
+	    slipstream.module {
+	       update = function (self)
+			   parts.engine.spark = self.input > 0
+		        end,
+	    }
+	 },
+
 	 -- Front brake control.
 
 	 frontbrake = slipstream.cascade {
 	    axis = 2,
 
-	    frames.node {
-	       input = 0,
-	       output = 0,
-
+	    slipstream.module {
 	       update = function (self)
 			   self.output = tonumber(options.brake) or
 			   math.clamp(-self.input *
@@ -683,12 +758,9 @@ local root = bodies.system {
 	    },
 
 	    slipstream.lag {lag = blueprint.frontbrakelag},
-	    slipstream.blackbox {entries = blackboxes[3]},
+	    slipstream.blackbox {channel = 3},
 	    
-	    frames.node {
-	       input = 0,
-	       output = 0,
-
+	    slipstream.module {
 	       update = function (self)
 			   parts.frontwheel.parent.motor = {0, self.input}
 		        end,
@@ -700,10 +772,7 @@ local root = bodies.system {
 	 rearbrake = slipstream.cascade {
 	    axis = 2,
 
-	    frames.node {
-	       input = 0,
-	       output = 0,
-
+	    slipstream.module {
 	       update = function (self)
 			   self.output = tonumber(options.brake) or
 			   math.clamp(-self.input *
@@ -713,12 +782,9 @@ local root = bodies.system {
 	    },
 
 	    slipstream.lag {lag = blueprint.rearbrakelag},
-	    slipstream.blackbox {entries = blackboxes[4]},
-
-	    frames.node {
-	       input = 0,
-	       output = 0,
+	    slipstream.blackbox {channel = 4},
 
+	    slipstream.module {
 	       update = function (self)
 			   parts.rearwheel.parent.motor = {0, self.input}
 		        end,
@@ -726,38 +792,115 @@ local root = bodies.system {
 	 },
       },
 
+      -- Clutch control.
+
+      clutch = slipstream.cascade {
+      	 axis = 2,
+
+      	 slipstream.module {
+      	    update = function (self)
+      			self.output = 0
+      		     end,
+      	 },
+
+      	 slipstream.blackbox {channel = 5},
+
+      	 slipstream.module {
+      	    peak = blueprint.peak,
+
+      	    update = function (self)
+      			parts.clutch.motor = {0, self.peak * (1 - self.input)}
+      		     end,
+      	 },
+      },
+
       -- Gearbox control.
       
       gearbox = slipstream.cascade {
-	 frames.event {
-	    input = 0,
-	    output = 0,
-	    buttonpress = function (self, button, x, y)
-			     if button == 1 and
-				self.output < #parts.engine.gearbox then
-				self.output = self.output + 1
-			     end
+	 primary = blueprint.primary,
+	 gears = blueprint.gearbox,
 
-			     if button == 3 and self.output > 0 then
-				self.output = self.output - 1
+	 slipstream.module {
+
+	    frames.event {
+	       buttonpress = function (self, button)
+				if button == 1 and
+				   self.parent.output < #self.ancestry[2].gears then
+				   self.parent.output = self.parent.output + 1
+				end
+
+				if button == 3 and self.parent.output > 0 then
+				   self.parent.output = self.parent.output - 1
+				end
 			     end
-			  end
+	    }
 	 },
 
-	 slipstream.blackbox {entries = blackboxes[5]},
-
-	 frames.node {
-	    input = 0,
-	    output = 0,
+	 slipstream.blackbox {channel = 6},
 
+	 slipstream.module {
 	    update = function (self)
- 		  parts.engine.gear = self.input
-	       end,
+			local q = self.parent.gears[self.input]
+
+			parts.gearbox.gear = self.input
+			parts.gearbox.ratio = q and q * self.parent.primary
+		     end,
 	 },
       },
    },
 
-   display = widgets.display {
+   profiler = frames.node {
+      laps = {},
+
+      link = function (self)
+		dynamics.collision.probe =
+		   function (a, b)
+		      local track, sampler
+		      local last, sum, top, samples = -1, 0, 0, 0
+
+		      if a.istrack or b.istrack then
+			 track = a.istrack and a or b
+			 sampler = track.sampler
+
+			 self.step =
+			    function (self)
+			       local p = sampler[self.parent.rear.position]
+			       local now = p and p[1] or -1
+			       local speed = math.length(self.parent.rear.velocity)
+			       
+			       if last ~= now then
+				  if last == #track - 1 and 
+				     now == 0 then
+				     table.insert (self.laps,
+						   {configuration.time[1],
+						    sum / samples,
+						    top})
+				  end
+
+				  last, top, sum, samples = now, 0, 0, 0
+			       end
+
+			       top = speed > top and speed or top
+			       sum = sum + speed
+			       samples = samples + 1
+			    end
+			 
+			 dynamics.collision.probe = nil
+		      end
+		   end
+	     end,
+
+      unlink = function (self)
+		  -- Don't overwrite recorded profiles
+		  -- when replaying.
+
+		  if not session.laps then
+		     session.laps = self.laps
+		  end
+	       end
+   },
+
+   display = not options.nowidgets and widgets.display {
       size = {graphics.window[1] / graphics.window[2], 1},
       padding = {0.01, 0.01},
       align = {-1, -1},
@@ -848,11 +991,31 @@ local root = bodies.system {
 	    },
 
 	    [3] = widgets.layout {
-	       traverse = function (self)			  
-			     self.text = string.format ("<span font=\"Sans 20\" color=\"White\">Speed: %d</span>\n<span font=\"Sans 14\" color=\"White\">Lean: %.1f\194\176</span>\n<span font=\"Sans 14\" color=\"White\">Gear: %s</span>",
-							math.length(parts.chassis.velocity) / units.kilometersperhour(1),
-							parts.rearwheel.state[1] / units.degrees(1),
-							parts.engine.gear or "N")
+	       traverse = function (self)
+			     local text
+			     local laps = self.ancestry[4].profiler.laps
+
+			     text = string.format ("<span font=\"Sans 20\" color=\"White\">Speed: %d</span>\n<span font=\"Sans 14\" color=\"White\">Lean: %.1f\194\176</span>\n<span font=\"Sans 14\" color=\"White\">Gear: %s</span>",
+							 math.length(parts.rearframe.velocity) / units.kilometersperhour(1),
+							 parts.rearwheel.state[1] / units.degrees(1),
+							 parts.gearbox.gear == 0 and "N" or parts.gearbox.gear)
+
+			     if #laps > 0 then
+				local time
+				local m, s = math.modf((configuration.time[1] - laps[#laps][1]) / 60)
+				
+				-- Break up time into mm:ss.
+
+				if m > 0 then
+				   time = string.format ("%d' %.2f\"", m, s * 60)
+				else
+				   time = string.format ("%2.2f\"", s * 60)
+				end
+
+				text = text .. string.format ("\n<span font=\"Sans 14\" color=\"White\">Lap: %s</span>", time)
+			     end
+
+			     self.text = text
 			  end,
 	    },
 
@@ -925,68 +1088,32 @@ local root = bodies.system {
 	 
    link = function (self)
 	     parts = {
-		swingarm = self.rear,
-		rearwheel = self.rear.spindle.wheel,
+		swingarm = self.swingarm,
+		rearwheel = self.swingarm.spindle.wheel,
 		frontwheel = self.front.slider.legs.spindle.wheel,
 		steering = self.front.stem,
 		servo = self.front.servo,
-		pilot = self.main.pelvis.butt.waist.torso,
-		chassis = self.main,
-		engine = self.main.engine,
-		rearsuspension = self.rear.lowerpivot.arm.pivot.arm.leftpivot.lowershock.slider,
+		pilot = self.rear.waist.torso,
+		rearframe = self.rear,
+		engine = self.rear.engine,
+		gearbox = self.rear.engine.flywheel.clutch.mainshaft.gearbox,
+		clutch = self.rear.engine.flywheel.clutch,
+		rearsuspension = self.swingarm.lowerpivot.arm.pivot.arm.leftpivot.lowershock.slider,
 		frontsuspension = self.front.slider,
-		drive = self.main.countershaft.sprocket.chain,
+		finaldrive = self.rear.countershaft.sprocket.chain,
 		control = self.control,
 	     }
-
-	     -- Load the needed set of server pages.
-
-	     network.mime['/motorcycle'] = "text/html"
-	     network.pages['/motorcycle'] = resources.readfile "slipstream/motorcycle/home.lua"
-
-	     network.mime['/motorcycle/engine'] = "text/html"
-	     network.pages['/motorcycle/engine'] = resources.readfile "slipstream/motorcycle/engine.lua"
-
-	     network.mime['/motorcycle/tyre'] = "text/html"
-	     network.pages['/motorcycle/tyre'] = resources.readfile "slipstream/motorcycle/tyre.lua"
-
-	     network.mime['/motorcycle/vehicle'] = "text/html"
-	     network.pages['/motorcycle/vehicle'] = resources.readfile "slipstream/motorcycle/vehicle.lua"
-
-	     network.mime['/motorcycle/control'] = "text/html"
-	     network.pages['/motorcycle/control'] = resources.readfile "slipstream/motorcycle/control.lua"
 	  end,
 
    unlink = function (self)
 	       parts = nil
-
-	       -- Unload the model's server pages.
-
-	       network.mime['/motorcycle'] = nil
-	       network.pages['/motorcycle'] = nil
-
-	       network.mime['/motorcycle/engine'] = nil
-	       network.pages['/motorcycle/engine'] = nil
-
-	       network.mime['/motorcycle/tyre'] = nil
-	       network.pages['/motorcycle/tyre'] = nil
-
-	       network.mime['/motorcycle/vehicle'] = nil
-	       network.pages['/motorcycle/vehicle'] = nil
-
-	       network.mime['/motorcycle/control'] = nil
-	       network.pages['/motorcycle/control'] = nil
 	    end,
 }
 
 -- Setup collision response.
 
 dynamics.collision.vehicle = function (a, b)
-				-- print(a, b)
-
-				if a.iswheel or b.iswheel then
-				   return 3, 0.7, 0.2, physics.spring (141e3, 1500)
-				elseif a.isvehicle or b.isvehicle then
+				if a.isvehicle or b.isvehicle then
 			   -- print ("Other!")
 				   return 3, 0.4, 0.1
 				end
