@@ -19,5 +19,4 @@ resources.dofile "slipstream/controls/motorcycle-largesports.lua"
 resources.dofile "slipstream/tyres/motorcycle-120-70.lua"
 resources.dofile "slipstream/tyres/motorcycle-180-55.lua"
 
-return resources.dofile "slipstream/motorcycle/model.lua"
-
+blueprint.model = "motorcycle"
