@@ -35,15 +35,43 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
   for more details.</p>
 
   <?lua
+    require "automotive"
+
     local wheel
     local F_x0, F_y0, M_z0 = {}, {}, {}
     local F_x, F_y, F_xy, M_z = {}, {}, {}, {}
 
     -- Make a couple of plots.
 
-    wheel = query.wheel == "rear" and parts.rearwheel
-       or parts.frontwheel
+    if query.wheel == "rear" then
+       wheel = automotive.wheel {
+	    resistance = blueprint.rearresistance,
+	    radii = blueprint.rearradii,
+	    elasticity = blueprint.rearelasticity,
+	    load = blueprint.rearload,
+	    relaxation = blueprint.rearrelaxation,
 
+	    longitudinal = blueprint.rearlongitudinal,  	    
+	    lateral = blueprint.rearlateral,
+	    moment = blueprint.rearmoment,
+
+	    scaling = blueprint.rearscaling,
+       }
+    else
+       wheel = automotive.wheel {
+	    resistance = blueprint.frontresistance,
+	    radii = blueprint.frontradii,
+	    elasticity = blueprint.frontelasticity,
+	    load = blueprint.frontload,
+	    relaxation = blueprint.frontrelaxation,
+
+	    longitudinal = blueprint.frontlongitudinal,  	    
+	    lateral = blueprint.frontlateral,
+	    moment = blueprint.frontmoment,
+
+	    scaling = blueprint.frontscaling,
+       }
+    end
     -- F_x0.
 
     for i = 1, 3 do
@@ -573,7 +601,71 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
        return html
      ?>
   </table>
+
+  <table class="configuration">
+    <tr><th colspan="4">Scaling factors</th></tr>
+    <tr>
+      <td>Peak longitudinal friction coefficient (&lambda;<sub>&mu;,x</sub>):</td>
+      <td><input type="text"
+		 name="blueprint.<?lua return query.wheel ?>scaling"
+		 value="<?lua return string.format ("%g", blueprint[query.wheel .. 'scaling'][1]) ?>" /></td>
+
+      <td>Peak lateral friction coefficient (&lambda;<sub>&mu;,y</sub>):</td>
+      <td><input type="text"
+		 name="blueprint.<?lua return query.wheel ?>scaling"
+		 value="<?lua return string.format ("%g", blueprint[query.wheel .. 'scaling'][2]) ?>" /></td>
+    </tr>
+
+    <tr>
+      <td>Brake slip stiffness (&lambda;<sub>Kx&kappa;</sub>):</td>
+      <td><input type="text"
+		 name="blueprint.<?lua return query.wheel ?>scaling"
+		 value="<?lua return string.format ("%g", blueprint[query.wheel .. 'scaling'][3]) ?>" /></td>
+
+      <td>Cornering stiffness (&lambda;<sub>Ky&alpha;</sub>):</td>
+      <td><input type="text"
+		 name="blueprint.<?lua return query.wheel ?>scaling"
+		 value="<?lua return string.format ("%g", blueprint[query.wheel .. 'scaling'][4]) ?>" /></td>
+    </tr>
+
+    <tr>
+      <td>Camber force stiffness (&lambda;<sub>Ky&gamma;</sub>):</td>
+      <td><input type="text"
+		 name="blueprint.<?lua return query.wheel ?>scaling"
+		 value="<?lua return string.format ("%g", blueprint[query.wheel .. 'scaling'][5]) ?>" /></td>
+
+      <td>Pneumatic trail (&lambda;<sub>t</sub>):</td>
+      <td><input type="text"
+		 name="blueprint.<?lua return query.wheel ?>scaling"
+		 value="<?lua return string.format ("%g", blueprint[query.wheel .. 'scaling'][6]) ?>" /></td>    
+    </tr>
+
+    <tr>      
+      <td>Camber torque stiffness (&lambda;<sub>Kz&gamma;</sub>):</td>
+      <td><input type="text"
+		 name="blueprint.<?lua return query.wheel ?>scaling"
+		 value="<?lua return string.format ("%g", blueprint[query.wheel .. 'scaling'][7]) ?>" /></td>
+
+      <td>Rolling resistance (&lambda;<sub>My</sub>):</td>
+      <td><input type="text"
+		 name="blueprint.<?lua return query.wheel ?>scaling"
+		 value="<?lua return string.format ("%g", blueprint[query.wheel .. 'scaling'][8]) ?>" /></td>
+    </tr>
+
+    <tr>
+      <td>Radial stiffness:</td>
+      <td><input type="text"
+		 name="blueprint.<?lua return query.wheel ?>scaling"
+		 value="<?lua return string.format ("%g", blueprint[query.wheel .. 'scaling'][9]) ?>" /></td>
+
+      <td>Radial damping:</td>
+      <td><input type="text"
+		 name="blueprint.<?lua return query.wheel ?>scaling"
+		 value="<?lua return string.format ("%g", blueprint[query.wheel .. 'scaling'][10]) ?>" /></td>
+    </tr>
+  </table>
+
   <hr/>
-  <?lua return fragments.endconfiguration {apply = true, save = true} ?>
+  <?lua return fragments.endconfiguration {apply = true, save = true, export = true} ?>
   <hr/>
 </body>
