@@ -422,13 +422,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
         </td>
     </tr>
     <tr><td>Steering geometry</td>
-        <td><input pre = "Math.round($ / Math.PI * 180 * 10) / 10"
-                   post = "$ / 180 * Math.PI"
+        <td><input pre="Math.round($ / Math.PI * 180 * 10) / 10"
+                   post="$ / 180 * Math.PI"
                    type="text"
                    name="blueprint.headangle"
 	           value="<?lua return string.format ("%g", blueprint.headangle) ?>" /> &deg head angle with a travel of
-            <input pre = "Math.round($ / Math.PI * 180 * 10) / 10"
-                   post = "$ / 180 * Math.PI"
+            <input pre="Math.round($ / Math.PI * 180 * 10) / 10"
+                   post="$ / 180 * Math.PI"
                    type="text"
                    name="blueprint.steeringtravel"
 	           value="<?lua return string.format ("%g", blueprint.steeringtravel) ?>" /> &deg.</td>
@@ -448,11 +448,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 					  blueprint.points[6][1]) * 1e3)
      ?> mm</td></tr>
     <?lua
+     require "moremath"
+
      local html
      local v, p, q, u, d
 
      v = {-math.sin(blueprint.headangle),
-		math.cos(blueprint.headangle)}
+	  math.cos(blueprint.headangle)}
      p = {blueprint.points[2][1], blueprint.points[2][3]}
      
      -- Calculate fork offset (distance of front wheel
@@ -684,7 +686,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
                    post="$ / 1e3"
                    type="text"
                    name="blueprint.sprockets"
-	           value="<?lua return string.format ("%g", parts.drive.radii[2]) ?>" /> mm</td>
+	           value="<?lua return string.format ("%g", blueprint.sprockets[2]) ?>" /> mm</td>
     </tr>
   </table>
   <table class="configuration">
@@ -715,6 +717,5 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
     </tr>
   </table>
   <hr/>
-  <?lua return fragments.endconfiguration {apply = true, save = true} ?>
-  <hr/>
+  <?lua return fragments.endconfiguration {apply = true, save = true, export = true} ?>
 </body>
