@@ -19,19 +19,61 @@ require "toon"
 require "moretextures"
 require "transforms"
 
-return toon.flat {
-   color = textures.periodic (textures.checkerboard (512, 512, 2)),
+-- Setup collision response.
 
-   bodies.environment {
-      automotive.racetrack {
-	 position = {-5e3, 0, 0},
-	 orientation = transforms.euler (0, 0, 0),
+if layout.scaling then
+   dynamics.collision.track = function (a, b)
+				 if a.iswheel or b.iswheel then
+				    assert (a.iswheel)
 
-	 istrack = true,
+				   -- print (a, b)
 
-	 scale = {10, 10},
-	 
-	 [1] = {1e4, 5e3, 5e3, 0, 0, 0},
-      }
+				    return 1, unpack (layout.scaling)
+				 end
+			      end
+end
+
+local L, S = 1e4, 300
+
+return nature.atmosphere {
+   size = {1024, 512},
+
+   turbidity = 3,
+
+   rayleigh = math.scale({6.95e-06, 1.18e-05, 2.44e-05}, 1),
+   mie = 7e-5,
+
+   sun = {1.74, math.pi/4},
+
+   -- The environment.
+
+   ground = toon.flat {
+      color = textures.periodic (textures.checkerboard (512, 512, 2)),
+
+      surface = meshes.static {
+	 size = {4, 6},
+	 vertices = {-(L / 2), -(L / 2), 0,  0, 0, 1,  -S, -S,
+		     (L / 2), -(L / 2), 0,  0, 0, 1,  S, -S,
+		     (L / 2), (L / 2), 0,  0, 0, 1,  S, S,
+		     -(L / 2), (L / 2), 0,  0, 0, 1,  -S, S},
+
+	 indices = {0, 1, 2, 0, 2, 3}
+      } (),
+
+      environment = bodies.environment {
+	 bodies.plane {
+	 },
+
+	 automotive.racetrack {
+	    position = {-(L / 2), 0, 0},
+	    orientation = transforms.euler (0, 0, 0),
+	    
+	    istrack = true,
+
+	    scale = {10, 10},
+      
+	    [1] = {L, (L / 2), (L / 2), 0, 0, 0},
+	 }
+      },
    },
 }
